package main

import (
	"fmt"
	"io"
	"log"
	"net/http"
	"screenmark/screenmark/models"
	"time"

	"github.com/joho/godotenv"
	"github.com/lib/pq"
)

func main() {
	err := godotenv.Load()
	if err != nil {
		panic(err)
	}

	err = models.InitDatabase()
	if err != nil {
		panic(err)
	}

	listener := pq.NewListener(models.CONNECTION, time.Second, time.Second, func(event pq.ListenerEventType, err error) {
		if err != nil {
			panic(err)
		}
	})
	defer listener.Close()

	go func() {
		err := listener.Listen("new_image")
		if err != nil {
			panic(err)
		}

		select {
		case parameters := <-listener.Notify:
			log.Println("received notification, new image available: " + parameters.Extra)

			go func() {
				openAiClient, err := CreateOpenAiClient()
				if err != nil {
					panic(err)
				}

				image, err := models.GetImage(parameters.Extra)
				if err != nil {
					log.Println(err)
					return
				}

				imageInfo, err := openAiClient.GetImageInfo(image.ImageName, image.Image)
				if err != nil {
					log.Println(err)
					return
				}

				log.Printf("Info: %+v\n", imageInfo)
			}()
		}
	}()

	mux := http.NewServeMux()

	mux.HandleFunc("OPTIONS /image/{name}", func(w http.ResponseWriter, r *http.Request) {
		w.Header().Add("Access-Control-Allow-Origin", "*")
		w.Header().Add("Access-Control-Allow-Credentials", "*")
		w.Header().Add("Access-Control-Allow-Headers", "*")
	})

	mux.HandleFunc("POST /image/{name}", func(w http.ResponseWriter, r *http.Request) {
		imageName := r.PathValue("name")

		userId := r.Header.Get("userId")

		w.Header().Add("Access-Control-Allow-Origin", "*")
		w.Header().Add("Access-Control-Allow-Credentials", "*")
		w.Header().Add("Access-Control-Allow-Headers", "*")

		if len(imageName) == 0 {
			w.WriteHeader(http.StatusBadRequest)
			fmt.Fprintf(w, "You need to provide a name in the path")
			return
		}

		image, err := io.ReadAll(r.Body)

		if err != nil {
			w.WriteHeader(http.StatusBadRequest)
			fmt.Fprintf(w, "Couldnt read the image from the request body")
			return
		}

		err = models.SaveImage(userId, imageName, image)
		if err != nil {
			log.Println(err)
			w.WriteHeader(http.StatusBadRequest)
			fmt.Fprintf(w, "Could not save image to DB")
			return
		}
	})

	log.Println("Listening and serving.")

	http.ListenAndServe(":3040", mux)
}
